@@ -36,12 +36,6 @@
 #include <iostream>
 #include <iterator>
 
-/* This header have to be included before boost/regex.hpp header
-   If it is included after, there is an error
-   "undefined reference to CSHA1::Update (unsigned char const*, unsigned long)"
-*/
-#include "SHA1.h"
-
 #include <boost/foreach.hpp>
 #include <boost/regex.hpp>
 
@@ -50,6 +44,7 @@
 #include "imageio.h"
 #include "imagebuf.h"
 #include "imagebufalgo.h"
+#include "hash.h"
 
 OIIO_NAMESPACE_USING;
 
@@ -72,19 +67,33 @@ static bool compute_stats = false;
 static void
 print_sha1 (ImageInput *input)
 {
-    imagesize_t size = input->spec().image_bytes ();
-    if (size >= std::numeric_limits<size_t>::max()) {
-        printf ("    SHA1 digest: (unable to compute, image is too big)\n");
-        return;
+    SHA1 sha;
+    const ImageSpec &spec (input->spec());
+    if (spec.deep) {
+        // Special handling of deep data
+        DeepData dd;
+        if (! input->read_native_deep_image (dd)) {
+            printf ("    SHA-1: unable to compute, could not read image\n");
+            return;
+        }
+        // Hash both the sample counds and the data block
+        sha.appendvec (dd.nsamples);
+        sha.appendvec (dd.data);
+    } else {
+        imagesize_t size = input->spec().image_bytes (true /*native*/);
+        if (size >= std::numeric_limits<size_t>::max()) {
+            printf ("    SHA-1: unable to compute, image is too big\n");
+            return;
+        }
+        std::vector<unsigned char> buf((size_t)size);
+        if (! input->read_image (TypeDesc::UNKNOWN /*native*/, &buf[0])) {
+            printf ("    SHA-1: unable to compute, could not read image\n");
+            return;
+        }
+        sha.appendvec (buf);
     }
-    boost::scoped_array<unsigned char> buf(new unsigned char[(unsigned int)size]);
-    input->read_image (input->spec().format, &buf[0]);
-    CSHA1 sha;
-    sha.Update ((const unsigned char *)&buf[0], (unsigned int) size);
-    sha.Final ();
-    std::string digest;
-    sha.ReportHashStl (digest, CSHA1::REPORT_HEX_SHORT);
-    printf ("    SHA-1: %s\n", digest.c_str());
+
+    printf ("    SHA-1: %s\n", sha.digest().c_str());
 }
 
 
@@ -173,15 +182,15 @@ print_stats (const std::string &filename,
              const ImageSpec &originalspec,
              int subimage=0, int miplevel=0, bool indentmip=false)
 {
+    PixelStats stats;
     const char *indent = indentmip ? "      " : "    ";
+
     ImageBuf input;
-    
     if (! read_input (filename, input, subimage, miplevel)) {
         std::cerr << "Stats: read error: " << input.geterror() << "\n";
         return;
     }
     
-    PixelStats stats;
     if (! computePixelStats (stats, input)) {
         printf ("%sStats: (unable to compute)\n", indent);
         return;
@@ -240,26 +249,49 @@ print_stats (const std::string &filename,
         printf ("%llu ", (unsigned long long)stats.finitecount[i]);
     }
     printf ("\n");
-    
-    std::vector<float> constantValues(input.spec().nchannels);
-    if(isConstantColor(input, &constantValues[0])) {
-        printf ("%sConstant: Yes\n", indent);
-        printf ("%sConstant Color: ", indent);
-        for (unsigned int i=0; i<constantValues.size(); ++i) {
-            print_stats_num (constantValues[i], maxval, false);
-            printf (" ");
+
+    if (input.deep()) {
+        const DeepData *dd (input.deepdata());
+        size_t npixels = dd->nsamples.size();
+        size_t totalsamples = 0, emptypixels = 0;
+        size_t maxsamples = 0, minsamples = std::numeric_limits<size_t>::max();
+        for (size_t p = 0;  p < npixels;  ++p) {
+            size_t c = dd->nsamples[p];
+            totalsamples += c;
+            if (c > maxsamples)
+                maxsamples = c;
+            if (c < minsamples)
+                minsamples = c;
+            if (c == 0)
+                ++emptypixels;
         }
-        print_stats_footer (maxval);
-        printf ("\n");
-    }
-    else {
-        printf ("%sConstant: No\n", indent);
-    }
-    
-    if(isMonochrome(input)) {
-        printf ("%sMonochrome: Yes\n", indent);
+        printf ("%sMin deep samples in any pixel : %llu\n", indent, (unsigned long long)minsamples);
+        printf ("%sMax deep samples in any pixel : %llu\n", indent, (unsigned long long)maxsamples);
+        printf ("%sAverage deep samples per pixel: %.2f\n", indent, double(totalsamples)/double(npixels));
+        printf ("%sTotal deep samples in all pixels: %llu\n", indent, (unsigned long long)totalsamples);
+        printf ("%sPixels with deep samples   : %llu\n", indent, (unsigned long long)(npixels-emptypixels));
+        printf ("%sPixels with no deep samples: %llu\n", indent, (unsigned long long)emptypixels);
     } else {
-        printf ("%sMonochrome: No\n", indent);
+        std::vector<float> constantValues(input.spec().nchannels);
+        if (isConstantColor(input, &constantValues[0])) {
+            printf ("%sConstant: Yes\n", indent);
+            printf ("%sConstant Color: ", indent);
+            for (unsigned int i=0; i<constantValues.size(); ++i) {
+                print_stats_num (constantValues[i], maxval, false);
+                printf (" ");
+            }
+            print_stats_footer (maxval);
+            printf ("\n");
+        }
+        else {
+            printf ("%sConstant: No\n", indent);
+        }
+    
+        if (isMonochrome(input)) {
+            printf ("%sMonochrome: Yes\n", indent);
+        } else {
+            printf ("%sMonochrome: No\n", indent);
+        }
     }
 }
 
@@ -409,7 +441,8 @@ print_info_subimage (int current_subimage, int max_subimages, ImageSpec &spec,
         if (spec.depth > 1)
             printf (" x %4d", spec.depth);
         int bits = spec.get_int_attribute ("oiio:BitsPerSample", 0);
-        printf (", %d channel, %s%s", spec.nchannels,
+        printf (", %d channel, %s%s%s", spec.nchannels,
+                spec.deep ? "deep " : "",
                 extended_format_name(spec.format, bits),
                 spec.depth > 1 ? " volume" : "");
         printf (" %s", input->format_name());
@@ -432,6 +465,9 @@ print_info_subimage (int current_subimage, int max_subimages, ImageSpec &spec,
                          boost::regex_search ("sha-1", field_re))) {
         if (filenameprefix)
             printf ("%s : ", filename.c_str());
+        // Before sha-1, be sure to point back to the highest-res MIP level
+        ImageSpec tmpspec;
+        input->seek_subimage (current_subimage, 0, tmpspec);
         print_sha1 (input);
     }
 
@@ -464,7 +500,6 @@ print_info (const std::string &filename, size_t namefieldlength,
             ImageInput *input, ImageSpec &spec,
             bool verbose, bool sum, long long &totalsize)
 {
-    bool printed = false;
     int padlen = std::max (0, (int)namefieldlength - (int)filename.length());
     std::string padding (padlen, ' ');
 
@@ -499,6 +534,8 @@ print_info (const std::string &filename, size_t namefieldlength,
         if (spec.depth > 1)
             printf (" x %4d", spec.depth);
         printf (", %d channel, ", spec.nchannels);
+        if (spec.deep)
+            printf ("deep ");
         if (spec.channelformats.size()) {
             for (size_t c = 0;  c < spec.channelformats.size();  ++c)
                 printf ("%s%s", c ? "/" : "",
@@ -523,7 +560,6 @@ print_info (const std::string &filename, size_t namefieldlength,
         if (! verbose && num_of_subimages == 1 && any_mipmapping)
             printf (" (+mipmap)");
         printf ("\n");
-        printed = true;
     }
 
     if (verbose && num_of_subimages != 1) {
@@ -599,7 +635,7 @@ main (int argc, const char *argv[])
 
     long long totalsize = 0;
     BOOST_FOREACH (const std::string &s, filenames) {
-        ImageInput *in = ImageInput::create (s.c_str(), "" /* searchpath */);
+        ImageInput *in = ImageInput::open (s.c_str());
         if (! in) {
             std::string err = geterror();
             if (err.empty())
@@ -607,16 +643,9 @@ main (int argc, const char *argv[])
             std::cerr << "iinfo: " << err << "\n";
             continue;
         }
-        ImageSpec spec;
-        if (in->open (s.c_str(), spec)) {
-            print_info (s, longestname, in, spec, verbose, sum, totalsize);
-            in->close ();
-        } else {
-            std::string err = in->geterror();
-            if (err.empty())
-                err = Strutil::format ("Could not open \"%s\"", s.c_str());
-            std::cerr << "iinfo: " << err << "\n";
-        }
+        ImageSpec spec = in->spec();
+        print_info (s, longestname, in, spec, verbose, sum, totalsize);
+        in->close ();
         delete in;
     }
 
