@@ -47,6 +47,7 @@ OIIO_NAMESPACE_USING;
 // Test ImageBuf::zero and ImageBuf::fill
 void test_zero_fill ()
 {
+    std::cout << "test zero_fill\n";
     const int WIDTH = 8;
     const int HEIGHT = 6;
     const int CHANNELS = 4;
@@ -113,6 +114,7 @@ void test_zero_fill ()
 // Test ImageBuf::crop
 void test_crop ()
 {
+    std::cout << "test crop\n";
     int WIDTH = 8, HEIGHT = 6, CHANNELS = 4;
     // Crop region we'll work with
     int xbegin = 3, xend = 5, ybegin = 0, yend = 4;
@@ -149,9 +151,77 @@ void test_crop ()
 
 
 
+void test_paste ()
+{
+    std::cout << "test paste\n";
+    // Create the source image, make it a gradient
+    ImageSpec Aspec (4, 4, 3, TypeDesc::FLOAT);
+    ImageBuf A ("A", Aspec);
+    for (ImageBuf::Iterator<float> it (A);  !it.done();  ++it) {
+        it[0] = float(it.x()) / float(Aspec.width-1);
+        it[1] = float(it.y()) / float(Aspec.height-1);
+        it[2] = 0.1f;
+    }
+
+    // Create destination image -- black it out
+    ImageSpec Bspec (8, 8, 3, TypeDesc::FLOAT);
+    ImageBuf B ("B", Bspec);
+    float gray[3] = { .1, .1, .1 };
+    ImageBufAlgo::fill (B, gray);
+
+    // Paste a few pixels from A into B -- include offsets
+    ImageBufAlgo::paste (B, 2, 2, 0, 1 /* chan offset */,
+                         A, ROI(1, 4, 1, 4));
+
+    // Spot check
+    float a[3], b[3];
+    B.getpixel (1, 1, 0, b);
+    OIIO_CHECK_EQUAL (b[0], gray[0]);
+    OIIO_CHECK_EQUAL (b[1], gray[1]);
+    OIIO_CHECK_EQUAL (b[2], gray[2]);
+
+    B.getpixel (2, 2, 0, b);
+    A.getpixel (1, 1, 0, a);
+    OIIO_CHECK_EQUAL (b[0], gray[0]);
+    OIIO_CHECK_EQUAL (b[1], a[0]);
+    OIIO_CHECK_EQUAL (b[2], a[1]);
+
+    B.getpixel (3, 4, 0, b);
+    A.getpixel (2, 3, 0, a);
+    OIIO_CHECK_EQUAL (b[0], gray[0]);
+    OIIO_CHECK_EQUAL (b[1], a[0]);
+    OIIO_CHECK_EQUAL (b[2], a[1]);
+}
+
+
+
+void test_channel_append ()
+{
+    std::cout << "test channel_append\n";
+    ImageSpec spec (2, 2, 1, TypeDesc::FLOAT);
+    ImageBuf A ("A", spec);
+    ImageBuf B ("B", spec);
+    float Acolor = 0.1, Bcolor = 0.2;
+    ImageBufAlgo::fill (A, &Acolor);
+    ImageBufAlgo::fill (B, &Bcolor);
+
+    ImageBuf R ("R");
+    ImageBufAlgo::channel_append (R, A, B);
+    OIIO_CHECK_EQUAL (R.spec().width, spec.width);
+    OIIO_CHECK_EQUAL (R.spec().height, spec.height);
+    OIIO_CHECK_EQUAL (R.nchannels(), 2);
+    for (ImageBuf::ConstIterator<float> r(R); !r.done(); ++r) {
+        OIIO_CHECK_EQUAL (r[0], Acolor);
+        OIIO_CHECK_EQUAL (r[1], Bcolor);
+    }
+}
+
+
+
 // Tests ImageBufAlgo::add
 void test_add ()
 {
+    std::cout << "test add\n";
     const int WIDTH = 8;
     const int HEIGHT = 8;
     const int CHANNELS = 4;
@@ -184,6 +254,7 @@ void test_add ()
 // Tests ImageBufAlgo::compare
 void test_compare ()
 {
+    std::cout << "test compare\n";
     // Construct two identical 50% grey images
     const int WIDTH = 10, HEIGHT = 10, CHANNELS = 3;
     ImageSpec spec (WIDTH, HEIGHT, CHANNELS, TypeDesc::FLOAT);
@@ -231,6 +302,8 @@ main (int argc, char **argv)
 {
     test_zero_fill ();
     test_crop ();
+    test_paste ();
+    test_channel_append ();
     test_add ();
     test_compare ();
     
