@@ -228,10 +228,6 @@ pvt::contiguize (const void *src, int nchannels,
         return _contiguize ((const float *)src, nchannels, 
                             xstride, ystride, zstride,
                             (float *)dst, width, height, depth);
-    case TypeDesc::DOUBLE :
-        return _contiguize ((const double *)src, nchannels, 
-                            xstride, ystride, zstride,
-                            (double *)dst, width, height, depth);
     case TypeDesc::INT8:
     case TypeDesc::UINT8 :
         return _contiguize ((const char *)src, nchannels, 
@@ -254,6 +250,10 @@ pvt::contiguize (const void *src, int nchannels,
         return _contiguize ((const long long *)src, nchannels, 
                             xstride, ystride, zstride,
                             (long long *)dst, width, height, depth);
+    case TypeDesc::DOUBLE :
+        return _contiguize ((const double *)src, nchannels, 
+                            xstride, ystride, zstride,
+                            (double *)dst, width, height, depth);
     default:
         ASSERT (0 && "OpenImageIO::contiguize : bad format");
         return NULL;
@@ -269,24 +269,21 @@ pvt::convert_to_float (const void *src, float *dst, int nvals,
     switch (format.basetype) {
     case TypeDesc::FLOAT :
         return (float *)src;
+    case TypeDesc::UINT8 :
+        convert_type ((const unsigned char *)src, dst, nvals);
+        break;
     case TypeDesc::HALF :
         convert_type ((const half *)src, dst, nvals);
         break;
-    case TypeDesc::DOUBLE :
-        convert_type ((const double *)src, dst, nvals);
+    case TypeDesc::UINT16 :
+        convert_type ((const unsigned short *)src, dst, nvals);
         break;
     case TypeDesc::INT8:
         convert_type ((const char *)src, dst, nvals);
         break;
-    case TypeDesc::UINT8 :
-        convert_type ((const unsigned char *)src, dst, nvals);
-        break;
     case TypeDesc::INT16 :
         convert_type ((const short *)src, dst, nvals);
         break;
-    case TypeDesc::UINT16 :
-        convert_type ((const unsigned short *)src, dst, nvals);
-        break;
     case TypeDesc::INT :
         convert_type ((const int *)src, dst, nvals);
         break;
@@ -299,6 +296,9 @@ pvt::convert_to_float (const void *src, float *dst, int nvals,
     case TypeDesc::UINT64 :
         convert_type ((const unsigned long long *)src, dst, nvals);
         break;
+    case TypeDesc::DOUBLE :
+        convert_type ((const double *)src, dst, nvals);
+        break;
     default:
         ASSERT (0 && "ERROR to_float: bad format");
         return NULL;
@@ -400,8 +400,7 @@ pvt::convert_from_float (const float *src, void *dst, size_t nvals,
 
 bool
 convert_types (TypeDesc src_type, const void *src, 
-               TypeDesc dst_type, void *dst, int n,
-               int alpha_channel, int z_channel)
+               TypeDesc dst_type, void *dst, int n)
 {
     // If no conversion is necessary, just memcpy
     if ((src_type == dst_type || dst_type.basetype == TypeDesc::UNKNOWN)) {
@@ -409,39 +408,29 @@ convert_types (TypeDesc src_type, const void *src,
         return true;
     }
 
-    // Conversions are via a temporary float array
-    bool use_tmp = false;
-    boost::scoped_array<float> tmp;
-    float *buf;
-    if (src_type == TypeDesc::FLOAT) {
-        buf = (float *) src;
-    } else {
-        tmp.reset (new float[n]);  // Will be freed when tmp exists its scope
-        buf = tmp.get();
-        use_tmp = true;
+    if (dst_type == TypeDesc::TypeFloat) {
+        // Special case -- converting non-float to float
+        pvt::convert_to_float (src, (float *)dst, n, src_type);
+        return true;
     }
 
-    if (use_tmp) {
-        // Convert from 'src_type' to float (or nothing, if already float)
-        switch (src_type.basetype) {
-        case TypeDesc::UINT8 : convert_type ((const unsigned char *)src, buf, n); break;
-        case TypeDesc::UINT16 : convert_type ((const unsigned short *)src, buf, n); break;
-        case TypeDesc::FLOAT : convert_type ((const float *)src, buf, n); break;
-        case TypeDesc::HALF :  convert_type ((const half *)src, buf, n); break;
-        case TypeDesc::DOUBLE : convert_type ((const double *)src, buf, n); break;
-        case TypeDesc::INT8 :  convert_type ((const char *)src, buf, n);  break;
-        case TypeDesc::INT16 : convert_type ((const short *)src, buf, n); break;
-        case TypeDesc::INT :   convert_type ((const int *)src, buf, n); break;
-        case TypeDesc::UINT :  convert_type ((const unsigned int *)src, buf, n);  break;
-        case TypeDesc::INT64 : convert_type ((const long long *)src, buf, n); break;
-        case TypeDesc::UINT64 : convert_type ((const unsigned long long *)src, buf, n);  break;
-        default:         return false;  // unknown format
+    // Conversion is to a non-float type
+
+    boost::scoped_array<float> tmp;   // In case we need a lot of temp space
+    float *buf = (float *)src;
+    if (src_type != TypeDesc::TypeFloat) {
+        // If src is also not float, convert through an intermediate buffer
+        if (n <= 4096)  // If < 16k, use the stack
+            buf = ALLOCA (float, n);
+        else {
+            tmp.reset (new float[n]);  // Freed when tmp exists its scope
+            buf = tmp.get();
         }
+        pvt::convert_to_float (src, buf, n, src_type);
     }
 
-    // Convert float to 'dst_type' (just a copy if dst is float)
+    // Convert float to 'dst_type'
     switch (dst_type.basetype) {
-    case TypeDesc::FLOAT :  memcpy (dst, buf, n * sizeof(float));       break;
     case TypeDesc::UINT8 :  convert_type (buf, (unsigned char *)dst, n);  break;
     case TypeDesc::UINT16 : convert_type (buf, (unsigned short *)dst, n); break;
     case TypeDesc::HALF :   convert_type (buf, (half *)dst, n);   break;
@@ -460,6 +449,17 @@ convert_types (TypeDesc src_type, const void *src,
 
 
 
+// Deprecated version -- keep for link compatibiity
+bool
+convert_types (TypeDesc src_type, const void *src, 
+               TypeDesc dst_type, void *dst, int n,
+               int alpha_channel, int z_channel)
+{
+    return convert_types (src_type, src, dst_type, dst, n);
+}
+
+
+
 bool
 convert_image (int nchannels, int width, int height, int depth,
                const void *src, TypeDesc src_type,
