@@ -1525,7 +1525,7 @@ ImageCacheImpl::getstats (int level) const
             std::sort (files.begin(), files.end(), bytesread_compare);
             out << "  Top files by bytes read:\n";
             for (int i = 0;  i < std::min<int> (topN, files.size());  ++i) {
-                if (files[i]->broken())
+                if (files[i]->broken() || !files[i]->validspec())
                     continue;
                 out << Strutil::format ("    %d   %6.1f MB (%4.1f%%)  ", i+1,
                                         files[i]->bytesread()/1024.0/1024.0,
@@ -1535,7 +1535,7 @@ ImageCacheImpl::getstats (int level) const
             std::sort (files.begin(), files.end(), iotime_compare);
             out << "  Top files by I/O time:\n";
             for (int i = 0;  i < std::min<int> (topN, files.size());  ++i) {
-                if (files[i]->broken())
+                if (files[i]->broken() || !files[i]->validspec())
                     continue;
                 out << Strutil::format ("    %d   %9s (%4.1f%%)   ", i+1,
                                         Strutil::timeintervalformat (files[i]->iotime()).c_str(),
@@ -1546,7 +1546,7 @@ ImageCacheImpl::getstats (int level) const
             out << "  Files with slowest I/O rates:\n";
             int n = 0;
             BOOST_FOREACH (const ImageCacheFileRef &file, files) {
-                if (file->broken())
+                if (file->broken() || !file->validspec())
                     continue;
                 if (file->iotime() < 0.25)
                     continue;
@@ -2346,10 +2346,9 @@ ImageCacheImpl::get_tile (ustring filename, int subimage, int miplevel,
     y = spec.y + ytile * spec.tile_height;
     z = spec.z + ztile * spec.tile_depth;
     TileID id (*file, subimage, miplevel, x, y, z);
-    ImageCacheTileRef tile;
-    if (find_tile_main_cache (id, tile, thread_info)) {
+    if (find_tile(id, thread_info)) {
+        ImageCacheTileRef tile(thread_info->tile);
         tile->_incref();   // Fake an extra reference count
-        tile->use ();
         return (ImageCache::Tile *) tile.get();
     } else {
         return NULL;
@@ -2419,6 +2418,8 @@ ImageCacheImpl::invalidate (ustring filename)
         tilemutex_holder (NULL);
     }
 
+    ustring fingerprint = file->fingerprint();
+
     {
         ic_write_lock fileguard (m_filemutex);
         file->invalidate ();
@@ -2427,7 +2428,7 @@ ImageCacheImpl::invalidate (ustring filename)
     // Remove the fingerprint corresponding to this file
     {
         spin_lock lock (m_fingerprints_mutex);
-        FilenameMap::iterator f = m_fingerprints.find (filename);
+        FilenameMap::iterator f = m_fingerprints.find (fingerprint);
         if (f != m_fingerprints.end())
             m_fingerprints.erase (f);
     }
