@@ -86,18 +86,8 @@ ImageRec::ImageRec (ImageRec &img, int subimage_to_copy,
             if (writable || img.pixels_modified() || !copy_pixels) {
                 // Make our own copy of the pixels
                 ib = new ImageBuf (img.name(), srcspec);
-                if (copy_pixels) {
-                    ImageBuf::ConstIterator<float> src (srcib);
-                    ASSERT (src.rawptr());
-                    ImageBuf::Iterator<float> dst (*ib);
-                    int nchans = ib->nchannels();
-                    while (! src.done()) {
-                        for (int c = 0;  c < nchans;  ++c)
-                            dst[c] = src[c];
-                        ++src;
-                        ++dst;
-                    }
-                }
+                if (copy_pixels)
+                    ib->copy_pixels (srcib);
             } else {
                 // The other image is not modified, and we don't need to be
                 // writable, either.
@@ -164,9 +154,11 @@ ImageRec::ImageRec (const std::string &name, int nsubimages,
         m_subimages[s].m_miplevels.resize (miplevels[s]);
         m_subimages[s].m_specs.resize (miplevels[s]);
         for (int m = 0;  m < miplevels[s];  ++m) {
-            ImageBuf *ib = new ImageBuf (name, specs[specnum]);
+            ImageBuf *ib = specs ? new ImageBuf (name, specs[specnum])
+                                 : new ImageBuf (name);
             m_subimages[s].m_miplevels[m].reset (ib);
-            m_subimages[s].m_specs[m] = specs[specnum];
+            if (specs)
+                m_subimages[s].m_specs[m] = specs[specnum];
             ++specnum;
         }
     }
@@ -181,7 +173,8 @@ ImageRec::read ()
         return true;
     static ustring u_subimages("subimages"), u_miplevels("miplevels");
     int subimages = 0;
-    if (! m_imagecache->get_image_info (ustring(name()), 0, 0, u_subimages,
+    ustring uname (name());
+    if (! m_imagecache->get_image_info (uname, 0, 0, u_subimages,
                                         TypeDesc::TypeInt, &subimages)) {
         std::cerr << "ERROR: file \"" << name() << "\" not found.\n";
         return false;  // Image not found
@@ -190,13 +183,23 @@ ImageRec::read ()
 
     for (int s = 0;  s < subimages;  ++s) {
         int miplevels = 0;
-        m_imagecache->get_image_info (ustring(name()), s, 0, u_miplevels,
+        m_imagecache->get_image_info (uname, s, 0, u_miplevels,
                                       TypeDesc::TypeInt, &miplevels);
         m_subimages[s].m_miplevels.resize (miplevels);
         m_subimages[s].m_specs.resize (miplevels);
         for (int m = 0;  m < miplevels;  ++m) {
+            // Force a read now for reasonable-sized first images in the
+            // file. This can greatly speed up the multithread case for
+            // tiled images by not having multiple threads working on the
+            // same image lock against each other on the file handle.
+            // We guess that "reasonable size" is 50 MB, that's enough to
+            // hold a 2048x1536 RGBA float image.  Larger things will 
+            // simply fall back on ImageCache.
+            bool forceread = (s == 0 && m == 0 &&
+                              m_imagecache->imagespec(uname,s,m)->image_bytes() < 50*1024*1024);
             ImageBuf *ib = new ImageBuf (name(), m_imagecache);
-            bool ok = ib->read (s, m);
+            bool ok = ib->read (s, m, forceread,
+                                TypeDesc::FLOAT /* force float */);
             ASSERT (ok);
             m_subimages[s].m_miplevels[m].reset (ib);
             m_subimages[s].m_specs[m] = ib->spec();
