@@ -57,6 +57,8 @@ public:
     bool updatemode;
     int threads;
     std::string full_command_line;
+    std::string printinfo_metamatch;
+    std::string printinfo_nometamatch;
 
     // Output options
     TypeDesc output_dataformat;
@@ -84,25 +86,9 @@ public:
     int return_value;                        // oiiotool command return code
     ColorConfig colorconfig;                 // OCIO color config
 
-    Oiiotool ()
-        : verbose(false), noclobber(false), allsubimages(false),
-          printinfo(false), printstats(false), hash(false),
-          updatemode(false),
-          threads(0),
-          output_dataformat(TypeDesc::UNKNOWN), output_bitspersample(0),
-          output_scanline(false), output_tilewidth(0), output_tileheight(0),
-          output_compression(""), output_quality(-1),
-          output_planarconfig("default"),
-          output_adjust_time(false), output_autocrop(true),
-          diff_warnthresh(1.0e-6f), diff_warnpercent(0),
-          diff_hardwarn(std::numeric_limits<float>::max()),
-          diff_failthresh(1.0e-6f), diff_failpercent(0),
-          diff_hardfail(std::numeric_limits<float>::max()),
-          imagecache(NULL),
-          return_value (EXIT_SUCCESS),
-          m_pending_callback(NULL), m_pending_argc(0)
-    {
-    }
+    Oiiotool ();
+
+    void clear_options ();
 
     // Force img to be read at this point.
     void read (ImageRecRef img);
@@ -144,6 +130,8 @@ public:
         return r;
     }
 
+    ImageRecRef top () { return curimg; }
+
     void error (const std::string &command, const std::string &explanation);
 
 private:
@@ -214,8 +202,9 @@ public:
     // for each subimages is in miplevels[0..nsubimages-1], and specs[]
     // contains the specs for all the MIP levels of subimage 0, followed
     // by all the specs for the MIP levels of subimage 1, and so on.
+    // If spec == NULL, the IB's will not be fully allocated/initialized.
     ImageRec (const std::string &name, int nsubimages,
-              const int *miplevels, const ImageSpec *specs);
+              const int *miplevels, const ImageSpec *specs=NULL);
 
     // Number of subimages
     int subimages() const { return (int) m_subimages.size(); }
@@ -304,6 +293,7 @@ struct print_info_options {
     bool compute_sha1;
     bool compute_stats;
     std::string metamatch;
+    std::string nometamatch;
     size_t namefieldlength;
 
     print_info_options ()
