@@ -454,11 +454,11 @@ PSDInput::mode_channel_count[] =
 // Obligatory material to make this a recognizeable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-DLLEXPORT ImageInput *psd_input_imageio_create () { return new PSDInput; }
+OIIO_EXPORT ImageInput *psd_input_imageio_create () { return new PSDInput; }
 
-DLLEXPORT int psd_imageio_version = OIIO_PLUGIN_VERSION;
+OIIO_EXPORT int psd_imageio_version = OIIO_PLUGIN_VERSION;
 
-DLLEXPORT const char * psd_input_extensions[] = {
+OIIO_EXPORT const char * psd_input_extensions[] = {
     "psd", "pdd", "psb", NULL
 };
 
@@ -477,7 +477,7 @@ bool
 PSDInput::open (const std::string &name, ImageSpec &newspec)
 {
     m_filename = name;
-    m_file.open (name.c_str(), std::ios::binary);
+    Filesystem::open (m_file, name, std::ios::binary);
     if (!m_file.is_open ()) {
         error ("\"%s\": failed to open file", name.c_str());
         return false;
