@@ -36,6 +36,7 @@
 #include "dassert.h"
 #include "typedesc.h"
 #include "imageio.h"
+#include "filesystem.h"
 #include "fmath.h"
 #include "strutil.h"
 #include "sysutil.h"
@@ -116,11 +117,11 @@ private:
 // Obligatory material to make this a recognizeable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-DLLEXPORT ImageOutput *rla_output_imageio_create () { return new RLAOutput; }
+OIIO_EXPORT ImageOutput *rla_output_imageio_create () { return new RLAOutput; }
 
-// DLLEXPORT int rla_imageio_version = OIIO_PLUGIN_VERSION;   // it's in rlainput.cpp
+// OIIO_EXPORT int rla_imageio_version = OIIO_PLUGIN_VERSION;   // it's in rlainput.cpp
 
-DLLEXPORT const char * rla_output_extensions[] = {
+OIIO_EXPORT const char * rla_output_extensions[] = {
     "rla", NULL
 };
 
@@ -175,7 +176,7 @@ RLAOutput::open (const std::string &name, const ImageSpec &userspec,
     close ();  // Close any already-opened file
     m_spec = userspec;  // Stash the spec
 
-    m_file = fopen (name.c_str(), "wb");
+    m_file = Filesystem::fopen (name, "wb");
     if (! m_file) {
         error ("Could not open file \"%s\"", name.c_str());
         return false;
