@@ -40,12 +40,26 @@
 #include "imageio.h"
 #include "thread.h"
 #include "strutil.h"
+#include "filesystem.h"
 #include "fmath.h"
 
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 
+// General TIFF information:
+// TIFF 6.0 spec: 
+//     http://partners.adobe.com/public/developer/en/tiff/TIFF6.pdf
+// Other Adobe TIFF docs:
+//     http://partners.adobe.com/public/developer/tiff/index.html
+// Adobe extensions to allow 16 (and 24) bit float in TIFF (ugh, not on
+// their developer page, only on Chris Cox's web site?):
+//     http://chriscox.org/TIFFTN3d1.pdf
+// Libtiff:
+//     http://remotesensing.org/libtiff/
+
+
+
 // Helper struct for constructing tables of TIFF tags
 struct TIFF_tag_info {
     int tifftag;       // TIFF tag used for this info
@@ -79,6 +93,7 @@ public:
     TIFFInput () { init(); }
     virtual ~TIFFInput () { close(); }
     virtual const char * format_name (void) const { return "tiff"; }
+    virtual bool valid_file (const std::string &filename) const;
     virtual bool open (const std::string &name, ImageSpec &newspec);
     virtual bool open (const std::string &name, ImageSpec &newspec,
                        const ImageSpec &config);
@@ -130,8 +145,10 @@ private:
         }
     }
 
-    // Read tags from the current directory of m_tif and fill out spec
-    void readspec ();
+    // Read tags from the current directory of m_tif and fill out spec.
+    // If read_meta is false, assume that m_spec already contains valid
+    // metadata and should not be cleared or rewritten.
+    void readspec (bool read_meta=true);
 
     // Convert planar separate to contiguous data format
     void separate_to_contig (int n, const unsigned char *separate,
@@ -255,11 +272,11 @@ private:
 // Obligatory material to make this a recognizeable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-DLLEXPORT ImageInput *tiff_input_imageio_create () { return new TIFFInput; }
+OIIO_EXPORT ImageInput *tiff_input_imageio_create () { return new TIFFInput; }
 
-// DLLEXPORT int tiff_imageio_version = OIIO_PLUGIN_VERSION; // it's in tiffoutput.cpp
+// OIIO_EXPORT int tiff_imageio_version = OIIO_PLUGIN_VERSION; // it's in tiffoutput.cpp
 
-DLLEXPORT const char * tiff_input_extensions[] = {
+OIIO_EXPORT const char * tiff_input_extensions[] = {
     "tiff", "tif", "tx", "env", "sm", "vsm", NULL
 };
 
@@ -281,6 +298,27 @@ my_error_handler (const char *str, const char *format, va_list ap)
 
 
 
+bool
+TIFFInput::valid_file (const std::string &filename) const
+{
+    FILE *file = Filesystem::fopen (filename, "r");
+    if (! file)
+        return false;  // needs to be able to open
+    unsigned short magic[2] = { 0, 0 };
+    size_t numRead = fread (magic, sizeof(unsigned short), 2, file);
+    fclose (file);
+    if (numRead != 2)  // fread failed
+    	return false;
+    if (magic[0] != TIFF_LITTLEENDIAN && magic[0] != TIFF_BIGENDIAN)
+        return false;  // not the right byte order
+    if ((magic[0] == TIFF_LITTLEENDIAN) != littleendian())
+        swap_endian (&magic[1], 1);
+    return (magic[1] == 42 /* Classic TIFF */ ||
+            magic[1] == 43 /* Big TIFF */);
+}
+
+
+
 bool
 TIFFInput::open (const std::string &name, ImageSpec &newspec)
 {
@@ -325,6 +363,11 @@ TIFFInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
         return true;
     }
 
+    // If we're emulating a MIPmap, only resolution is allowed to change
+    // between MIP levels, so if we already have a valid level in m_spec,
+    // we don't need to re-parse metadata, it's guaranteed to be the same.
+    bool read_meta = !(m_emulate_mipmap && m_tif && m_subimage >= 0);
+
     if (! m_tif) {
         // Use our own error handler to keep libtiff from spewing to stderr
         lock_guard lock (lasterr_mutex);
@@ -333,7 +376,12 @@ TIFFInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
     }
 
     if (! m_tif) {
+#ifdef _WIN32
+        std::wstring wfilename = Strutil::utf8_to_utf16 (m_filename);
+        m_tif = TIFFOpenW (wfilename.c_str(), "rm");
+#else
         m_tif = TIFFOpen (m_filename.c_str(), "rm");
+#endif
         if (m_tif == NULL) {
             error ("Could not open file: %s",
                    lasterr.length() ? lasterr.c_str() : m_filename.c_str());
@@ -345,7 +393,7 @@ TIFFInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
     m_next_scanline = 0;   // next scanline we'll read
     if (TIFFSetDirectory (m_tif, subimage)) {
         m_subimage = subimage;
-        readspec ();
+        readspec (read_meta);
         newspec = m_spec;
         if (newspec.format == TypeDesc::UNKNOWN) {
             error ("No support for data format of \"%s\"", m_filename.c_str());
@@ -450,7 +498,7 @@ static const TIFF_tag_info exif_tag_table[] = {
 
 
 void
-TIFFInput::readspec ()
+TIFFInput::readspec (bool read_meta)
 {
     uint32 width = 0, height = 0, depth = 0;
     unsigned short nchans = 1;
@@ -459,7 +507,23 @@ TIFFInput::readspec ()
     TIFFGetFieldDefaulted (m_tif, TIFFTAG_IMAGEDEPTH, &depth);
     TIFFGetFieldDefaulted (m_tif, TIFFTAG_SAMPLESPERPIXEL, &nchans);
 
-    m_spec = ImageSpec ((int)width, (int)height, (int)nchans);
+    if (read_meta) {
+        // clear the whole m_spec and start fresh
+        m_spec = ImageSpec ((int)width, (int)height, (int)nchans);
+    } else {
+        // assume m_spec is valid, except for things that might differ
+        // between MIP levels
+        m_spec.width = (int)width;
+        m_spec.height = (int)height;
+        m_spec.depth = (int)depth;
+        m_spec.full_x = 0;
+        m_spec.full_y = 0;
+        m_spec.full_z = 0;
+        m_spec.full_width = (int)width;
+        m_spec.full_height = (int)height;
+        m_spec.full_depth = (int)depth;
+        m_spec.nchannels = (int)nchans;
+    }
 
     float x = 0, y = 0;
     TIFFGetField (m_tif, TIFFTAG_XPOSITION, &x);
@@ -516,8 +580,10 @@ TIFFInput::readspec ()
             m_spec.set_format (TypeDesc::UINT16);
         else if (sampleformat == SAMPLEFORMAT_INT)
             m_spec.set_format (TypeDesc::INT16);
-        else if (sampleformat == SAMPLEFORMAT_IEEEFP)
-            m_spec.set_format (TypeDesc::HALF); // not to spec, but why not?
+        else if (sampleformat == SAMPLEFORMAT_IEEEFP) {
+            m_spec.set_format (TypeDesc::HALF);
+            // Adobe extension, see http://chriscox.org/TIFFTN3d1.pdf
+        }
         else
             m_spec.set_format (TypeDesc::UNKNOWN);
         break;
@@ -542,6 +608,12 @@ TIFFInput::readspec ()
         break;
     }
 
+    // If we've been instructed to skip reading metadata, because it is
+    // guaranteed to be identical to what we already have in m_spec,
+    // skip everything following.
+    if (! read_meta)
+        return;
+
     // Use the table for all the obvious things that can be mindlessly
     // shoved into the image spec.
     for (int i = 0;  tiff_tag_table[i].name;  ++i)
@@ -705,7 +777,12 @@ TIFFInput::readspec ()
         // I'm not sure what state TIFFReadEXIFDirectory leaves us.
         // So to be safe, close and re-seek.
         TIFFClose (m_tif);
+#ifdef _WIN32
+        std::wstring wfilename = Strutil::utf8_to_utf16 (m_filename);
+        m_tif = TIFFOpenW (wfilename.c_str(), "rm");
+#else
         m_tif = TIFFOpen (m_filename.c_str(), "rm");
+#endif
         TIFFSetDirectory (m_tif, m_subimage);
 
         // A few tidbits to look for
